import 'package:flutter/material.dart';

class ShakeNotifier extends ChangeNotifier {
  void shake() {
    notifyListeners();
  }
}

class ShakerBox extends StatefulWidget {

  final ShakeNotifier shakeNotifier;
  final Widget child;
  final double shakeDelta;
  const ShakerBox(this.shakeNotifier, this.child, {Key key, this.shakeDelta = 4}): super(key: key);

  @override
  _State createState() {
    return _State();
  }
}

class _State extends State<ShakerBox> with TickerProviderStateMixin{

  AnimationController _controller;

  @override
  void initState() {
    super.initState();
    widget.shakeNotifier.addListener(_shakeHandler);

    _controller = AnimationController(
      value: 0,
      duration: Duration(milliseconds: 200),
      vsync: this)
    ;
  }

  void _shakeHandler() {
    if (_controller.status == AnimationStatus.dismissed) {
      _controller.forward();
    } else if (_controller.status == AnimationStatus.completed
    || _controller.status == AnimationStatus.forward) {
      _controller.reset();
      _controller.forward();
    }
  }

  @override
  Widget build(BuildContext context) {
    return _Shaker(
      animation: _controller,
      child: widget.child,
      shakeDelta: widget.shakeDelta,
    );
  }

  @override
  void dispose() {
    widget.shakeNotifier.removeListener(_shakeHandler);
    super.dispose();
  }
}

class _Shaker extends AnimatedWidget {
  const _Shaker({
    Key key,
    Animation<double> animation,
    this.shakeDelta = 4,
    this.child,
  }) : super(key: key, listenable: animation);

  final Widget child;
  final double shakeDelta;

  Animation<double> get animation => listenable;

  double get translateX {
    double _shakeDelta = shakeDelta;
    final double t = animation.value;
    if (t <= 0.25)
      return -t * _shakeDelta;
    else if (t < 0.75)
      return (t - 0.5) * _shakeDelta;
    else
      return (1.0 - t) * 4.0 * _shakeDelta;
  }

  @override
  Widget build(BuildContext context) {
    return Transform(
      transform: Matrix4.translationValues(translateX, 0.0, 0.0),
      child: child,
    );
  }
}